class EncryptedSecret(str):
    """Encrypted version of a DecryptedSecret"""
    pass

class DecryptedSecret(str):
    """Decrypted version of a EncryptedSecret"""
    pass

class EnvString(str):
    pass
